// Gutierrez, Cesareo
// cgutie@essex.ac.uk
// CC484 Assignment.December 2002

import java.util.*;
import java.lang.*;

/**
 * This class implements most of the dirty job of the algorithm. It has been
 * tried to leave in the main class (NoGoods) a comprehensive code of the
 * implementation (as the pseudo-code proposed by Edward Tsang in his book) and
 * implement here more low abstract level code.
 *
 * @version 1.0 10/12/2002
 * @author Cesareo Gutierrez
 */
public class CompoundLabel implements Cloneable{

  /** the values given to each variable **/
  int[] m_labels;

  /**
  * Constructor
  * It sets up initial values. One value of -1 means, that variable has not
  * been given any value yet.
  */
  public CompoundLabel() {
    m_labels = new int[NoGoods.NUMBERQUEENS];
    Arrays.fill(m_labels, -1);
  }

  /**
  * findoutConflictSet
  * It creates a compound label with each variable and the one to study. If
  * they do not pass the constraints check there is a conflict between them, so
  * this variable is added to the conflict set.
  *
  * @param x   row of the cell to study
  * @param y   column of the cell to study
  * @param C   the constraints
  * @return    all the variables in conflict with x,y cell
  */
  public Vector findoutConflictSet(int x, int y, Constraints C){
    Vector conflictSet = new Vector();
    //For each variable with a given value
    for (int i = 0; i < m_labels.length; i++) {
      if(m_labels[i] == -1) continue;
      //It creates a compoundLabel with each variable and the one to study
      CompoundLabel tmp_comp_label = new CompoundLabel();
      tmp_comp_label.addCompoundLabel(x,y);
      tmp_comp_label.addCompoundLabel(i, m_labels[i]);
      //If They do not check the constraints there is a conflict between them.
      if (!C.checkConstraints(tmp_comp_label)){
        conflictSet.add(new Integer(i));
      }
    }
    return conflictSet;
  }

  /**
  * identifyMinimalCoveringSet
  * This is a recursive method. The main behaviour is: erase one variable of
  * the candidate to be minimal covering set and checks if it is still covering
  * the conflictSet. If it is, the algorithm does a recursive call with this
  * new candidate. Otherwise, it tries erasing all the variables (only once at
  * the same time) to identify all the minimum covering sets. Although, it is
  * a recursive function, It does not need a local copy of the labels because
  * the algorithm restore the state properly in order to not interact with
  * previos executions of the function in the running stack.
  *
  * @see CompoundLabel#checkCoverSet
  *
  * @param conflictSet the conflictSet to be covered
  * @return            a vector with all the minimum covering sets found
  */
  public Vector identifyMinimalCoveringSet(Vector[] conflictSet){
    Vector result = new Vector();
    boolean is_minimum = true;
    //For each variable
    for (int i = 0; i < m_labels.length; i++) {
      //Erase one of the variables
      if(m_labels[i] == -1) continue;
      int last_value = m_labels[i];
      m_labels[i] = -1;
      //Test if it is still coversing the conflict sense
      if(this.checkCoverSet(conflictSet)){
        is_minimum = false;
        result.addAll(this.identifyMinimalCoveringSet(conflictSet));
      }
      //Restore the variable value
      m_labels[i] = last_value;
    }
    //I am a minimum covering set
    if (is_minimum){
      result.add(this.clone());
    }
    return result;
  }


  /**
  * checkCoverSet
  * For each cell in the conflict set and for each queen that it is attacking it:
  * check if at least there is one in the candidate(this.m_labels), If there is
  * none this is no a covering set.
  *
  * @param conflictSet the conflictSet to be covered
  * @return            a vector with all the minimum covering sets found
  */
  public boolean checkCoverSet(Vector[] conflictSet){
    boolean this_cell_is_ok;
    //For each cell in the conflict set
    for (int i = 0; i < conflictSet.length; i++) {
      this_cell_is_ok = false;
      //And for each queen that it is attacking it
      for (Iterator j = conflictSet[i].iterator(); j.hasNext(); ) {
        int item = ((Integer)j.next()).intValue();
        //check if at least there is one in the candidate(this)
        if(m_labels[item] != -1){
          this_cell_is_ok = true;
          break;
        }
      }
      //If there is none this is not a covering set
      if (!this_cell_is_ok)return false;
    }
    return true;
  }

  /**
  * isSubset
  * It checks if all the values in the variables of newCl are in this compound label
  *
  * @param newCl   compound label
  * @return        if this is a subset of newCl
  */
  public boolean isSubset(CompoundLabel newCl){
    //If all the values in the variables of newCl are in this compound label
    for (int i = 0; i < m_labels.length; i++) {
      if (newCl.m_labels[i] != m_labels[i])
        return false;
    }
    return true;
  }

  /**
  * addCompoundLabel
  * It gives the value y to the variable x
  *
  * @param x  the variable
  * @param y  the value
  */
  public void addCompoundLabel(int x, int y){
    m_labels[x] = y;
  }

 /**
  * isThereAnyEqual
  * It checks if two variables has the same given value
  *
  * @return        if two variables has the same given value
  */
  public boolean isThereAnyEqual(){
    //It makes a temp copy of the values
    int[] tmp_labels = (int[])m_labels.clone();
    //It sorts the temp copy
    Arrays.sort(tmp_labels);
    //It tries to find to consecutive equal numbers
    for (int i = 0; i < (tmp_labels.length-1); i++) {
      if(tmp_labels[i] == -1) continue;
      if(tmp_labels[i] == tmp_labels[i+1])
        return true;
    }
    return false;
  }

  /**
  * isThereEqualDistances
  * It checks if two variables has the same distance in rows and columns
  *
  * @return        if two variables has the same distance in rows and columns
  */
  public boolean isThereEqualDistances(){
    //For each variable
    for (int i = 0; i < m_labels.length; i++) {
      //If it has not given a value continue
      if (m_labels[i] == -1) continue;
      //For each of the variables
      for (int j = i+1; j < m_labels.length; j++) {
         if(m_labels[j] == -1) continue;
         if(j-i == Math.abs(m_labels[j]-m_labels[i]))
           return true;
      }
    }
    return false;
  }

  /**
  * isEmpty
  * It checks if there is no more variables unlabelled. One value of -1 means,
  * that variable has not been given any value yet.
  *
  * @return  if there is no more variables unlabelled.
  */
  public boolean isEmpty(){
    for (int i = 0; i < m_labels.length; i++) {
      if(m_labels[i] != -1)
        return false;
    }
    return true;
  }

   /**
  * toString
  * It creates a string with a text description in the 8-queens problem domain.
  *
  * @return        a text description in the 8-queens problem domain.
  */
   public String toString(){
    StringBuffer strBuf = new StringBuffer();
    strBuf.append("------------------\n");
    for (int i = 0; i < m_labels.length; i++) {
      strBuf.append("|");
      for (int j = 0; j < m_labels.length; j++) {
        if(m_labels[i] == j){
          strBuf.append("Q ");
        }else{
          strBuf.append("  ");
        }
      }
      strBuf.append("|\n");
    }
    strBuf.append("------------------");
    return strBuf.toString();
  }

  public String toBriefString(){
    //Create a stringbuffer
    StringBuffer strbuf = new StringBuffer();
    //Text the queens one by one
    for (int i = 0; i < m_labels.length; i++) {
      int j = m_labels[i];
      if(j == -1) continue;
      strbuf.append(Integer.toString(i)+(char)(j+65)+" ");
    }
    return strbuf.toString();
  }

  /**
  * clone
  * It ensures that the algorithm has unique local variables.
  *
  * @return a clone of the actual object.
  */
  public Object clone(){
    CompoundLabel obj=null;
    try {
      obj =  (CompoundLabel) super.clone();
      obj.m_labels = (int[])this.m_labels.clone();
    }catch (Exception ex) {
      ex.printStackTrace();
    }
    return obj;
  }

  /**
  * getQueen
  * It is only used for drawing on the gui
  *
  * @param  the variable x
  * @return the value of the variable x
  */
  //
  public int getQueen(int x){
    return m_labels[x];
  }
}
