// Gutierrez, Cesareo
// cgutie@essex.ac.uk
// CC484 Assignment.December 2002

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

/**
 * This class implements the Graphical User Interface of the application. It is
 * composed of a JPanel with two buttons, a JTextArea and a JLayeredPane. The
 * JPanel has the buttons goOn and a buttonGroup. Each times the algorithm stops
 * to show the temporary results waits for the user to push this button in
 * order to go on with the algorithm. The buttonGroup let the user choose among
 * three different options: stop in each single step, stop only when a no good
 * set is found or stop never. In all the cases stop when it reaches a solution.
 * This makes the visualization of the progress of the algorithm better to
 * follow. The JTextArea displays the NoGood algorithms and solutions
 * found. The JLayeredPane has two boards. The one in the left shows each
 * movement that the algorithm does. The one in the right shows only the
 * noGood combinations found.
 *
 * @see NoGoods
 *
 * @version 1.0 10/12/2002
 * @author Cesareo Gutierrez
 */
public class QueensGUI extends JFrame implements ActionListener{

  /** The graphical dimmension fo the cell **/
  static int DIM_CELDA = 50;

  /** GUI Components **/
  private JPanel m_buttonPanel;
  private JButton m_goOn;
  private JRadioButton m_stopAlways, m_stopOnly, m_stopNever;
  private ButtonGroup m_radioGroup;
  private JTextArea m_outputArea;
  private JLayeredPane m_board;
  private ImageIcon m_board_image= new ImageIcon("board.gif");
  private ImageIcon m_queen_image= new ImageIcon("queen.gif");


  /** two modes stopping **/
  boolean m_stopAlwaysBool = true;
  boolean m_stopNeverBool = false;
  boolean m_stopOnlyBool = false;
  boolean m_waiting = false;

  /** the labels that encapsulates the images of the queens **/
  JLabel m_labels[][]= new JLabel[NoGoods.NUMBERQUEENS][NoGoods.NUMBERQUEENS];
  Vector m_labels2= new Vector();

  /**
  * Constructor
  * Create the graphical user interface and set up the initial values.
  */
   public QueensGUI() {
    super( "CC484 Assignment 2002 : Learning NoGoods Algorithm : 8 queens problem" );

    // create buttons
    m_goOn = new JButton("Go on");
    m_goOn.addActionListener(this);
    m_stopAlways = new JRadioButton("Stop Always",true);
    m_stopAlways.addActionListener(this);
    m_stopOnly = new JRadioButton("Only stop in no goods",false);
    m_stopOnly.addActionListener(this);
    m_stopNever = new JRadioButton("Stop Never(Only Solutions)",false);
    m_stopNever.addActionListener(this);

    // create logical relationship between JRadioButtons
    m_radioGroup = new ButtonGroup();
    m_radioGroup.add(m_stopAlways);
    m_radioGroup.add(m_stopNever);
    m_radioGroup.add(m_stopOnly);

    // create button panel and add the buttons
    m_buttonPanel = new JPanel();
    m_buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
    m_buttonPanel.add(m_goOn);
    m_buttonPanel.add(m_stopAlways);
    m_buttonPanel.add(m_stopOnly);
    m_buttonPanel.add(m_stopNever);

    //create TextArea and JLayeredPane
    m_outputArea = new JTextArea();
    m_board = new JLayeredPane();
    m_board.setPreferredSize(new Dimension(NoGoods.NUMBERQUEENS*DIM_CELDA,NoGoods.NUMBERQUEENS*DIM_CELDA));
    m_board.setBorder(BorderFactory.createTitledBorder("8-queens board"));

    //Add components to the container
    Container container = getContentPane();
    container.add( m_buttonPanel, BorderLayout.NORTH );
    container.add( m_board,BorderLayout.CENTER );
    container.add( m_outputArea , BorderLayout.SOUTH );

    //Add the board draw
    JLabel dibujo = new JLabel(m_board_image);
    dibujo.setBounds(10, 20, NoGoods.NUMBERQUEENS*DIM_CELDA,NoGoods.NUMBERQUEENS*DIM_CELDA);
    m_board.add(dibujo,new Integer(0));
    JLabel dibujo2 = new JLabel(m_board_image);
    dibujo2.setBounds(500, 20, NoGoods.NUMBERQUEENS*DIM_CELDA,NoGoods.NUMBERQUEENS*DIM_CELDA);
    m_board.add(dibujo2,new Integer(0));

    setSize(930,530);
    setVisible(true);
  }

  /**
  * main
  * Setup the initial values and call the algorithm to start
  *
  * @param args[1] if it exists the initial values are chosen randomly
  */
  public static void main(String[] args) {
    //It creates the GUI
    QueensGUI queensGUI1 = new QueensGUI();
    queensGUI1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE );

    //It checks if the user wants random initial values
    if(args.length == 1){
      MyArray.RANDOM = true;
    }

    //It creates and setup the initial values. The values are set up in the
    //constructor of MyArray
    MyArray Z = new MyArray(NoGoods.NUMBERQUEENS);
    MyArray[] D = new MyArray[NoGoods.NUMBERQUEENS];
    for (int i = 0; i < NoGoods.NUMBERQUEENS; i++) {
      D[i] = new MyArray(NoGoods.NUMBERQUEENS);
    }
    Constraints C = new Constraints();

    //It creates the algorithm object and runs it
    NoGoods noGoods1 = new NoGoods(queensGUI1);
    noGoods1.learnNoGoodAlgorithm(Z,D,C);
  }


  /**
  * drawSolution
  * It draws a text description of the solution
  *
  * @param cl the solution
  */
  public void drawSolution(CompoundLabel cl){
    m_outputArea.setText("New Solution found: "+cl.toBriefString());
    while(m_waiting);
  }

  /**
  * drawQueen
  * It draws a queen in the board on the left
  *
  * @param x row
  * @param y column
  */
   public void drawQueen(int x, int y){
     //draw the queen in the board on the left
     addQueen(x,y,1);
     //Waits the user to push the button goOn
     m_waiting = true;
     if(m_stopAlwaysBool) while(m_waiting);
  }

  /**
  * drawNoGood
  * It draws a no good combination of queens in the board on the right
  *
  * @param cl the description of the nogoodset
  */
  public void drawNoGood(CompoundLabel cl){
    //It removes the last No good drawn
    removeBoard2();
    //Draw the queens one by one
    for (int i = 0; i < NoGoods.NUMBERQUEENS; i++) {
      int j = cl.getQueen(i);
      if (j == -1) continue;
      addQueen(i,j,2);
    }
    //Print a brief description of the no good set
    m_outputArea.setText("New NoGood Found: "+cl.toBriefString());
    //Waits the user to push the button goOn
    m_waiting = true;
    if(m_stopAlwaysBool || m_stopOnlyBool) while(m_waiting);
  }

  /**
  * removeBoard2
  * It removes all the queens of the board on the right
  */
  public void removeBoard2(){
    //It removes the queens of the board one by one
    for (Iterator i = m_labels2.iterator(); i.hasNext(); ) {
      JLabel item = (JLabel)i.next();
      //It updates the graphics and remove the queen
      item.setVisible(false);
      m_board.remove(item);
    }
  }
  /**
  * removeQueen
  * It removes the specified queen of the board in the left
  *
  * @param x row
  * @param y column
  */
  public void removeQueen(int x, int y){
    //It takes the JLabel that corresponds to that location
    JLabel reina = m_labels[x][y];
    //It upgrades the graphics and removes the queen
    reina.setVisible(false);
    m_board.remove(reina);
  }

  /**
  * addQueen
  * It draws the specified queen on the specified board
  *
  * @param x         row
  * @param y         column
  * @param num_board the queen will be drawn in that board
  */
  public void addQueen(int x,int y, int num_board){
    //It calculates the exact location on the screen
    int posx = 20+x*DIM_CELDA;
    int posy = 10+y*DIM_CELDA;
    if(num_board == 2) posy += 490;
    //It creates the JLabel
    JLabel reina = new JLabel(m_queen_image);
    reina.setBounds(posy, posx, DIM_CELDA, DIM_CELDA);
    //It safes where the queen are located
    if(num_board != 2) m_labels[x][y] = reina;
    else m_labels2.add(reina);
    //It adds the queen to the layered pane
    m_board.add(reina,new Integer(1));
  }

  /**
  * actionPerformed
  * It is called either when the goOn button is pressed or when the onlyNoGoods
  * checkbox state changes.
  *
  * @param actionEvent
  */
  public void actionPerformed( ActionEvent actionEvent){
    Object source = actionEvent.getSource();
    if(source == m_stopOnly){
      m_stopOnlyBool = true;
      m_stopAlwaysBool = false;
      m_stopNeverBool = false;
    }else if(source == m_stopAlways){
      m_stopOnlyBool = false;
      m_stopAlwaysBool = true;
      m_stopNeverBool = false;
    }else if(source == m_stopNever){
      m_stopOnlyBool = false;
      m_stopAlwaysBool = false;
      m_stopNeverBool = true;
    }else{
      if(m_waiting){
        m_waiting = false;
      }
    }
  }
}